#include <fstream>
#include <string>
#include <assert.h>
#include <time.h>


#include "image.hh"
#include "format_pgm.hh"
#include "format_ppm.hh"


#include <iostream>

static image::iterator static_it;


image::image():start(NULL),img_format()
{
}
image::image(const char* name):start(NULL),img_format(name)
{
  if (!img_format)
    {
      cerr << "Warning : Unrecognized format : " << name<< endl;
      return;
    }
  if (img_format.load(name,dim,width,height,start) <0)
    {
      cerr << "Warning : Unable to read : " << name<< endl;
      return;
    }
  width_size=dim*width;
  img_size=width_size*height;
}
image::image(int dim_img,int largeur,int hauteur):dim(dim_img),
						  width(largeur),
						  height(hauteur),
						  width_size(dim*width),
						  img_size(width_size*height)
{
  if(dim == 1)
    img_format = pgm_format();
  else
    img_format = ppm_format();
  
  start =  (Comp_rgb*) malloc(img_size);
  memset(start,0,img_size);
}
image::image(const image & img):start(NULL),
                                img_format(img.img_format),
                                dim(img.dim),
				width(img.width),
				height(img.height),
				width_size(img.width_size),
				img_size(img.img_size)
{
  if(img.start == NULL)
    return;
  start = (Comp_rgb*)malloc(img_size);
  memcpy(start,img.start,img_size);
}
image::~image()
{
  if(start != NULL)
    free(start);
}
void image::release()
{
    if(start != NULL)
      {
	free(start);
	start=NULL;
      }
    dim=width=height=width_size=img_size=0;
}
image &image::operator =(const image &img)
{
  img_format = img.img_format;
  dim = img.dim;
  width = img.width;
  height = img.height;
  width_size = img.width_size;
  img_size = img.img_size;

  if(start != NULL)
    free(start);
  start=NULL;
  if(img.start == NULL)
    return *this;
  start = (Comp_rgb*)malloc(img_size);
  memcpy(start,img.start,img_size);
  return *this;  
}
  /*******************************************/
 /* Fonctions de chargements et Sauvegarde */
/*****************************************/
int image::load(const char* name)
{
  Comp_rgb* start2;
  
  img_format=format(name);

  if(!img_format)
      return -1;

  if (img_format.load(name,dim,width,height,start2) <0)
      return -2;

  if(start)
    free(start);
  start = start2;
  width_size=dim*width;
  img_size=width_size*height;

  return 0;
}

int image::save(const char* name)
{
  if (!img_format)
    return -1;

  return img_format.save(name,dim,width,height,start);
}
Comp_rgb &image::operator()(int i,int j,int k)  
{
  if (i<0|| i>=height || j <0||j>width)
    throw image_error ("Invalid coordinates");
  if(k<0||k>=dim)
    throw image_error("Request component incompatible with dimension");

  return *(start +i*width_size+dim*j+k);
}
image::iterator& image::begin() const
{
  assert(start != NULL);
  image::iterator beg(*this);
  return static_it=beg;
}
image::iterator& image::end() const
{
  assert(start != NULL);
  image::iterator last(*this,beyond_image);
  return static_it=last;
}
image::iterator& image::last() const
{
  assert(start != NULL);
  image::iterator last(*this,last_pixel);
  return static_it=last;
}
image::iterator::iterator():begin(0),
			    dim(0),
			    width_size(0),
			    size(0),
			    current(0)

{
}
image::iterator::iterator(const image& img,position pos):begin(img.start),
							 dim(img.dim),
							 width_size(img.width_size),
							 size(img.img_size),
							 current(0)

{
  if(pos==first)
    return;
  if(pos== last_pixel)
    current=size-1;
  else
    current=size;
}
image::iterator::iterator(const iterator& iter):begin(iter.begin),
						      dim(iter.dim),
						      width_size(iter.width_size),
						      size(iter.size),
						      current(iter.current)

{
}
Comp_rgb& image::iterator::operator*()
{
  if(current <0 || current >= size)
    throw iteratorator_error("Attempt to acess to a non allocated area");

  return *(begin+current);
}
Comp_rgb& image::iterator::operator[](int k)
{
  if(k<0|| k>= dim)
    throw iteratorator_error("Request for a field incompatible with dim");
      
  if(current +k <0 || current +k>= size)
    throw iteratorator_error("Attempt to acess to a non allocated area");

  return *(begin+current+k);
}

image::iterator & image::iterator::operator++()
{
  current += dim;
  return (*this);
}

image::iterator  image::iterator::operator++(int nothing) 
{
  image::iterator tmp(*this);
  current += dim;
  return tmp;
}
image::iterator & image::iterator::operator+=(int pixel)
{
  current += dim*pixel;
  return (*this);
}

image::iterator & image::iterator::operator--()
{
  current -= dim;
  return (*this);
}
image::iterator  image::iterator::operator--(int nothing) 
{
  iterator tmp(*this);
  current -= dim;
  return tmp;
}
image::iterator & image::iterator::operator-=(int pixel)
{
  current -= dim*pixel;
  return (*this);
}
bool  image::iterator::operator<(const iterator & iter) const
{
  return current < iter.current;
}
bool  image::iterator::operator>(const iterator & iter) const
{
  return current > iter.current;
}
bool  image::iterator::operator<=(const iterator & iter) const
{
  return current <= iter.current;
}
bool image::iterator::operator>=(const iterator& iter )const
{
  return current >= iter.current;
}
bool image::iterator::operator==(const iterator &iter) const
{
  return current == iter.current;
}
bool image::iterator::operator!=(const iterator& iter) const
{
  return current != iter.current;
}
bool image::iterator::move_to(int i,int j)
{
  int dep(i*width_size+dim*j);
  
  if (dep <0 || dep > size)
    return false;
  current = dep;
  return true;
}
image::iterator image::iterator::operator+(int dep) const 
{
  if(current+dep <0 || current +dep >= size )
    throw iteratorator_error("Wrong offset of current image pointer");
  
  iterator tmp(*this);
  tmp.current+=dep;
  return tmp;
}
image::iterator image::iterator::operator-(int dep) const 
{
  return (*this)+(-dep);
}
image::iterator& image::iterator::operator =(iterator& iter)
{
  begin=iter.begin;
  dim=iter.dim;
  width_size=iter.width_size;
  size=iter.size;
  current=iter.current;

  return *this;
}

int image::Dim() const
{
  return dim;
}
int image::Height() const
{
  return height;
}
int image::Width() const
{
  return width;
}

ostream & operator << (ostream& o, image& img)
{
  
  return img.img_format.to_stream(o,img.dim,img.width,img.height,img.start);
}
