#ifndef IMAGE_H
#define IMAGE_H


#include <iostream>
#include "format.hh"

using namespace std;

class image_error
{
public:
  image_error(){}
  image_error(const char* messg)
  {
    cerr << messg<<endl;
  }
};
class iteratorator_error
{
public:
  iteratorator_error(){}
  iteratorator_error(const char* messg)
  {
    cerr << messg<<endl;
  }
};

typedef unsigned char Comp_rgb;
class image
{  
  Comp_rgb * start;
  format img_format;
  int dim;  /* image dim 1 or 3 */
  int width,height;
  int width_size, img_size;   /*nb cels of one line (Largeur*Dim),  nb image's cels   */

  bool valid(int,int);
public :
  image();
  image(const char*);
  /// dim, width, height
  image(int,int,int);
  image(const image&);
  ~image();
  void release();
  image& operator =(const image&);
  Comp_rgb& operator ()(int,int,int);
  int       load(const char*);
  int       save(const char*);  
  typedef enum {first,last_pixel,beyond_image} position;
  class iterator
  {
  protected:
    Comp_rgb * begin;
    int dim;
    int width_size;    
    int size;
    int current;
  public:
    iterator();
    iterator(const image&,position beg=first);
    iterator(const iterator&);

    Comp_rgb & operator *();
    Comp_rgb & operator[](int);


    virtual iterator & operator++();
    virtual iterator  operator++(int);
    virtual iterator & operator+=(int);
    virtual iterator & operator--();
    virtual iterator  operator--(int);
    virtual iterator & operator-=(int);

    virtual bool operator<(const iterator&) const;
    virtual bool operator>(const iterator&) const;
    virtual bool operator<=(const iterator&) const;
    virtual bool operator>=(const iterator&)const;
    bool operator==(const iterator&) const;
    bool operator!=(const iterator&) const;

    iterator operator +(int) const;
    iterator operator -(int) const ;
    iterator& operator =(iterator&);

    // line, column
    bool move_to(int,int);
  };
  iterator& begin() const;
  iterator& end() const;
  iterator& last() const;
  friend  ostream & operator << (ostream&, image&);
  int       Dim() const;
  int       Height() const;
  int       Width() const;

};
#endif /* IMAGE_H */



