#include "border_data_deriche.hh"

static border_data_deriche::border_value static_border_value;

border_data_deriche::border_data_deriche(image &I,double &d):max(0),alpha(d)
{
  init_deriche_deriv(alpha,I);
  grad_x=deriche_deriv_x_float(dim_x, dim_y);
  grad_y=deriche_deriv_y_float(dim_x, dim_y);
  close_deriche_deriv();
}
border_data_deriche::border_data_deriche(const border_data_deriche &b):data(b.data),
								       dim_x(b.dim_x),
								       dim_y(b.dim_y),
								       max(b.max),
								       alpha(b.alpha)
{
  int size = dim_x*dim_y;
  if (grad_x!=0)
    delete [] grad_x;
  grad_x = new float [size];
  memcpy(grad_x, b.grad_x, size*sizeof(float));
  if (grad_y!=0)
    delete [] grad_y;
  grad_y = new float [size];
  memcpy(grad_y, b.grad_y, size*sizeof(float));
}
border_data_deriche::~border_data_deriche()
{
  if (grad_x!=0)
    delete [] grad_x;
  if (grad_y!=0)
    delete [] grad_y;
  int i, nb=dim_x*(dim_y+1)+(dim_x+1)*dim_y;
  i=-nb;
  do
  {
    if ( data(i)!=NULL )
    {
      delete (border_value *)data[i];
      data[i] = NULL;
    }
  }
  while(++i);
  do
  {
    if ( data(nb)!=NULL )
    {
      delete (border_value *)data[nb];
      data[nb] = NULL;
    }
  }
  while(--nb);
}

border_data_deriche & border_data_deriche::operator=(const border_data_deriche &b)
{
  data = b.data;
  dim_x = b.dim_x;
  dim_y = b.dim_y;
  max = b.max;
  alpha = b.alpha;

  int size = dim_x*dim_y;
  if (grad_x!=0)
    delete [] grad_x;
  grad_x = new float [size];
  memcpy(grad_x, b.grad_x, size*sizeof(float));
  if (grad_y!=0)
    delete [] grad_y;
  grad_y = new float [size];
  memcpy(grad_y, b.grad_y, size*sizeof(float));
  return *this;
}

border_data_deriche::border_value::border_value():grad(0), cardinal(1)
{}
border_data_deriche::border_value::border_value(const border_value &b):grad(b.grad), cardinal(b.cardinal)
{}
border_data_deriche::border_value::border_value(float val):grad(val), cardinal(1)
{}
border_data_deriche::border_value & border_data_deriche::border_value::operator=(const border_value &b)
{
  grad = b.grad;
  cardinal = b.cardinal;
  return *this;
}
void border_data_deriche::border_value::reinit(float g,int c)
{
  grad = g;
  cardinal = c;
}
float border_data_deriche::border_value::give_gradient()
{
  return grad;
}
int border_data_deriche::border_value::give_cardinal()
{
  return cardinal;
}


border_data_deriche::border_value &border_data_deriche::operator() (long i) const
{
  border_value *pt = (border_value *)(data(i));
  if (pt==NULL)
  {
    static_border_value.reinit(0,0);
    return static_border_value;
  }
  return *pt;
}
void border_data_deriche::del(dart b)
{
  delete (border_value *)(data[b]);
  data[b]=NULL;
}

void border_data_deriche::init(combi_map_base &map,dart b)
{
  point p;
  move_type move;
  float tmp;

  map.give_pixel(b,p);
  move = map.give_move(b);
  (border_value *)(data[b]) = new border_value;
  if (move==left_move || move==right_move)
  {
    tmp = grad_y[dim_x*p.y+p.x];
    ((border_value *)(data[b]))->reinit(tmp, 1);
    if (tmp > max)
      max=tmp;
    return;
  }
  tmp = grad_x[dim_x*p.y+p.x];
  ((border_value *)(data[b]))->reinit(tmp, 1);
  if (tmp > max)
    max=tmp;
}
void border_data_deriche::merge(int level,dart D1,dart D2)
{
  int C;
  double M;
  border_value *b1, *b2;
  b1 = (border_value *)(data(D1));
  b2 = (border_value *)(data(D2));
  M =b1->give_gradient();
  M+=b2->give_gradient();
  C =b1->give_cardinal();
  C+=b2->give_cardinal();
  b1->reinit(M,C);
}
float border_data_deriche::give_max_gradient()
{
  return max;
}
