/** \file border_data_deriche.hh
    \brief File to manage the gradient of each vertex */

#ifndef __BORDER_DATA_DERICHE_HH__
#define __BORDER_DATA_DERICHE_HH__

#include <tab_sym_ptr.hh>
#include <string.h>
#include "border_data.hh"
#include "deriche.hh"

/** One class which store the gradient of each vertex */
class border_data_deriche: public border_data
{
private:
  // Array of pointor on the border value
  tab_sym_ptr data;
  // Picture array of the x-gradient
  float *grad_x;
  // Picture array of the y-gradient
  float *grad_y;
  // Width of the gradient picture
  int    dim_x;
  // Height of the gradient picture
  int    dim_y;
  // The maximum value of the gradient
  float max;
  // The alpha parmeter used for the deriche
  double alpha;

public:
  /// Constructor with parameters, the number of darts
  border_data_deriche(image &I,double &d);
  /// Copy constructor
  border_data_deriche(const border_data_deriche &);
  /// Destructor
  virtual ~border_data_deriche();
  /// Reaffectation operator
  border_data_deriche & operator=(const border_data_deriche &);


  /** Class that hold all the information of one border (associated to one dart)*/
  class border_value
  {
  private:
    // Gradient of the edge
    float grad;
    // Number of edges that are merged to the border
    int cardinal;

  public:
    /// Constructor
    border_value();
    /// Copy constructor
    border_value(const border_value &);
    /// Constructor with parameters, the gradient value of the border
    border_value(float);
    /// Destructor
    ~border_value()
    {}

    /// Reaffectation operator
    border_value & operator=(const border_value &);
    /// Reinitialiation function
    void reinit(float,int);

    /// Access to the sum of the gradient value of all the borders which were merged
    float give_gradient();
    /// Access to the cardinal of the border (number of border merged with it)
    int   give_cardinal();
  };

  /// Access an element of dataset
  border_value &operator() (long) const;
  /// Remove an element from the dataset
  void del(dart);

  /// Function to initialize the border_value of the dart
  void init(combi_map_base &,dart);
  /// Function to manage the border_value during the merge of two dart
  void merge(int,dart,dart);
  /// Access to the maximum value of the gradient
  float give_max_gradient();
};

#endif // __BORDER_DATA_DERICHE_HH__
