/** \file combi_map.hh
    \brief File for the combinatorial maps */

#ifndef __COMBI_MAP_HH__
#define __COMBI_MAP_HH__

#include <label.hh>


/** An enum for the orientation of the dart */
typedef enum{right_move,up_move,left_move,down_move} move_type;

/** Model class for the combinatorial maps */
class combi_map
{
public:
  /// A pur virtual function to access to the sigma of one dart of the combinatorial map
  virtual dart sigma(dart) const=0;
  /// gives the label of a vertex associated to a dart
  virtual vertex mu(dart) const =0;

  /// A pur virtual function which returns the dart d=sigma_1(b) such that sigma(d)=b
  // we use the relationship sigma^{-1}(b)=alpha(phi^{-1}(b))
  dart sigma_1(dart b)
  {
    dart b_prev, s=phi(b);
    b_prev=s;
    while(s != b)
      {
	b_prev=s;
	s=phi(s);
      }
    return alpha(b_prev);
  }
  /// A pure virtual function which  returns the phi successor of a
  /// dart sigma o alpha
  dart phi(dart b)
  {
    return sigma(alpha(b));
  }
  /// A pur virtual function to access to the alpha of one dart of the combinatorial map
  virtual dart alpha(dart) const=0;
  /// A pur virtual function that give the number of edge
  virtual int give_nb_edge() const =0 ;
  /// A pur virtual function that give the backround value
  virtual unsigned long give_background() const =0;
};

#endif // __COMBI_MAP_HH__
