/** \file combi_map_gen.hh
    \brief File for the generalised combinatorial maps */

#ifndef __COMBI_MAP_GEN__
#define __COMBI_MAP_GEN__

#include <list_label.hh>
#include <tab_dart.hh>
#include "combi_map.hh"

class pyramid;

typedef list_label::iterator vertex_iterator;

/** Class for the generalised combinatorial maps */
class combi_map_gen: public combi_map
{
public:
  /// pointor to the pyramid from which the map was extracted
  pyramid *pyr;
  /// Tab of dart that hold the sigma of the map
  tab_dart sig;
  /// Tab of dart that hold the alpha of each dart in the map
  tab_dart alph;
  /// Tab of dart that hold the corresponding vertex for each dart of the map
  tab_dart m;
  /// Tab of vertex that hold on dart for each vertex of the map
  tab_label m_1;
  /// List of the vertex of the map
  list_label vertex_set;
  /// Number of edge in the map
  int nb_edge;
  /// The backgroud value
  unsigned long background;

  /// Constructor
  combi_map_gen(pyramid *P):pyr(P),nb_edge(0),background(0)
  {}
  /// Virtual destructor
  virtual ~combi_map_gen()
  {}
  virtual vertex mu(dart d) const
  {
    return m(d);
  }
  /// A pur virtual function to access to the sigma of one dart of the combinatorial map
  virtual dart sigma(dart d) const
  {
    return sig(d);
  }
  /// Function to access to the alpha of one dart of the combinatorial map
  virtual dart alpha(dart d) const
  {
    return alph(d);
  }
  /// Function that give the number of edge
  virtual int give_nb_edge() const
  {
    return nb_edge;
  }
  /// Function that give the backround value
  unsigned long give_background() const
  {
    return background;
  }
  /// Function that give the number of edge
  int give_nb_vertices() const
  {
    return vertex_set.give_cardinal();
  }
};


#endif // __COMBI_MAP_GEN__
