#include "combinatorial_map.hh"
#include "pyramid.hh"

static combinatorial_map::segment::iterator res;

int combinatorial_map::give_neighbor_dart(vertex v)
{
  dart res, sig_b, b, open=0;
  res = sig_b = b = m_1(v);
  do
  {
    sig_b = sig(sig_b);
    if (open_parenthesis(sig_b) && open==0)
    {
      open = sig_b;
      res = sig(alph(sig_b));
      continue;
    }
    if (alph(sig_b)==open)
    {
      open = 0;
      continue;
    }
  }
  while (sig_b != b);
  return res;
}

int combinatorial_map::give_level(dart d)
{
  return pyr->give_level(d);
}

int combinatorial_map::give_width()
{
  return pyr->give_width();
}
int combinatorial_map::give_height()
{
  return pyr->give_height();
}
dart combinatorial_map::give_alpha_base(dart d)
{
  return pyr->give_alpha_base(d);
}
dart combinatorial_map::give_sigma_base(dart d)
{
  return pyr->give_sigma_base(d);
}
void combinatorial_map::give_point(dart d,point &p)
{
  pyr->give_pointel(d,p);
}
void combinatorial_map::give_pixel(dart d,point &p)
{
  pyr->give_pixel(d,p);
}
move_type combinatorial_map::give_move(dart d)
{
  return pyr->give_move_base(d);
}
vertex_iterator &combinatorial_map::give_vertex_first()
{
  return vertex_set.first();
}
vertex_iterator &combinatorial_map::give_vertex_last()
{
  return vertex_set.last();
}
vertex_iterator &combinatorial_map::give_vertex_end()
{
  return vertex_set.end();
}

dart_iterator &combinatorial_map::give_dart_first()
{
  return pyr->first_dart(level);
}
dart_iterator &combinatorial_map::give_dart_begin()
{
  return pyr->begin_dart(level);
}
dart_iterator &combinatorial_map::give_dart_last()
{
  return pyr->last_dart(level);
}
dart_iterator &combinatorial_map::give_dart_end()
{
  return pyr->end_dart(level);
}

void combinatorial_map::give_all_neighbors(vertex v,list_label &neighbors,list_label &included)
{
  dart sig_b, b, close_parenthesis;
  vertex v_mb;
  sig_b = b = give_neighbor_dart(v);
  do
  {
    sig_b = sig(sig_b);
    if (open_parenthesis(sig_b))
    {
      close_parenthesis = alph(sig_b);
      do
      {
        sig_b = sig(sig_b);
	v_mb = m(alph(sig_b));
        if (v_mb!=v)
	  included.insert(v_mb);
      }
      while (sig_b!=close_parenthesis);
      continue;
    }
    neighbors.insert(m(alph(sig_b)));
  }
  while (sig_b != b);
}
void combinatorial_map::give_neighbors(vertex v,list_label &neighbors)
{
  dart sig_b, b, close_parenthesis;
  sig_b = b = give_neighbor_dart(v);
  do
  {
    sig_b = sig(sig_b);
    if (open_parenthesis(sig_b))
    {
      close_parenthesis = alph(sig_b);
      do
      {
        sig_b = sig(sig_b);
      }
      while (sig_b!=close_parenthesis);
      continue;
    }
    neighbors.insert(m(alph(sig_b)));
  }
  while (sig_b != b);
}
void combinatorial_map::give_included(vertex v,list_label &included)
{
  dart sig_b, b, close_parenthesis;
  sig_b = b = give_neighbor_dart(v);
  vertex v_mb;
  do
  {
    sig_b = sig(sig_b);
    if (open_parenthesis(sig_b))
    {
      close_parenthesis = alph(sig_b);
      do
      {
        sig_b = sig(sig_b);
	v_mb = m(alph(sig_b));
        if (v_mb!=v)
	  included.insert(v_mb);
      }
      while (sig_b!=close_parenthesis);
      continue;
    }
  }
  while (sig_b != b);
}

void combinatorial_map::display()
{
  list_label::iterator iter=vertex_set.first();
  dart sig_b, b;
  while(iter<vertex_set.end())
  {
    cout << *iter << ": " << endl;
    sig_b = b = m_1(*iter);
    do
    {
      sig_b = sig(sig_b);

      segment seg(sig_b,alpha(sig_b),level,pyr);
      segment::iterator it = seg.first();
      cout << "    ";
      while(it!=seg.end())
      {
        point p, p1, p2;
	it.give_point(p);
	it.give_pixel(p1,p2);
	if (it.give_move()==0)
          cout << "_ " << *it << "(" << p.x << "," << p.y << ")=>R (" << p1.x << "," << p1.y << "|" << p2.x << "," << p2.y << ")";
	if (it.give_move()==1)
          cout << "_ " << *it << "(" << p.x << "," << p.y << ")=>U (" << p1.x << "," << p1.y << "|" << p2.x << "," << p2.y << ")";
	if (it.give_move()==2)
          cout << "_ " << *it << "(" << p.x << "," << p.y << ")=>L (" << p1.x << "," << p1.y << "|" << p2.x << "," << p2.y << ")";
	if (it.give_move()==3)
          cout << "_ " << *it << "(" << p.x << "," << p.y << ")=>D (" << p1.x << "," << p1.y << "|" << p2.x << "," << p2.y << ")";
	it++;
      }
      cout << endl;
    }
    while( sig_b!=b );
    cout << endl;
    iter++;
  }
}

void combinatorial_map::draw_border(image &Im, unsigned char *C,draw_type type)
{
  image::iterator it_im(Im);
  segment seg;
  segment_iterator it_seg;
  dart_iterator it_dart = pyr->first_dart(level);
  point p;
  dart b,m_b;
  vertex mu, mu_1;

  if (type==cm_draw_single)
  {
    while (it_dart<pyr->end_dart(level))
    {
      b = *it_dart++;
      mu=m(b);
      if ((unsigned)mu==background)
        continue;
      m_b = alph(b);
      mu_1 = m(m_b);
      if (mu==mu_1)
        continue;
      if (b<m_b && (unsigned)mu_1!=background)
        continue;
      seg=give_segment(b);
      it_seg = seg.first();
      do
      {
        pyr->give_pixel(*it_seg++,p);
        it_im.move_to(p.y, p.x);
        for(int i=Im.Dim();i--;)
          it_im[i] = C[i];
      }
      while (it_seg!=seg.end());
    }
    return;
  }
  if (type==cm_draw_both)
  {
    while (it_dart<pyr->end_dart(level))
    {
      b = *it_dart++;
      mu=m(b);
      if ((unsigned)mu==background)
        continue;
      if (mu==m(alph(b)))
        continue;
      seg=give_segment(b);
      it_seg = seg.first();
      do
      {
        pyr->give_pixel(*it_seg++,p);
        it_im.move_to(p.y, p.x);
        for(int i=Im.Dim();i--;)
          it_im[i] = C[i];
      }
      while (it_seg!=seg.end());
    }
    return;
  }
  if (type==cm_draw_all_single)
  {
    while (it_dart<pyr->end_dart(level))
    {
      b = *it_dart++;
      if ((unsigned)m(b)==background)
        continue;
      m_b = alph(b);
      if (b<m_b && (unsigned)m(m_b)!=background)
        continue;
      seg=give_segment(b);
      it_seg = seg.first();
      do
      {
        pyr->give_pixel(*it_seg++,p);
        it_im.move_to(p.y, p.x);
        for(int i=Im.Dim();i--;)
          it_im[i] = C[i];
      }
      while (it_seg!=seg.end());
    }
    return;
  }
  while (it_dart<pyr->end_dart(level))
  {
    b = *it_dart++;
    if ((unsigned)m(b)==background)
      continue;
    seg=give_segment(b);
    it_seg = seg.first();
    do
    {
      pyr->give_pixel(*it_seg++,p);
      it_im.move_to(p.y, p.x);
      for(int i=Im.Dim();i--;)
	it_im[i] = C[i];
    }
    while (it_seg!=seg.end());
  }
}

void combinatorial_map::draw_vertex_border(vertex v, image &Im, unsigned char *C,draw_type type)
{
  image::iterator it_im(Im);
  segment seg;
  segment_iterator it_seg;
  point p;
  dart b,sig_b,m_b;
  if ((unsigned)v==background)
    return;
  if (type==cm_draw_single)
  {
    sig_b = b = m_1(v);
    do
    {
      sig_b = sig(sig_b);
      if (m(alph(sig_b))==v)
        continue;
      seg=give_segment(sig_b);
      it_seg = seg.first();
      do
      {
        pyr->give_pixel(*it_seg++,p);
        it_im.move_to(p.y, p.x);
        for(int i=Im.Dim();i--;)
          it_im[i] = C[i];
      }
      while (it_seg!=seg.end());
    }
    while(sig_b!=b);
    return;
  }

  if (type==cm_draw_both)
  {
    sig_b = b = m_1(v);
    do
    {
      sig_b = sig(sig_b);
      if (m(alph(sig_b))==v)
        continue;
      seg=give_segment(sig_b);
      it_seg = seg.first();
      do
      {
        pyr->give_pixel(*it_seg++,p);
        it_im.move_to(p.y, p.x);
        for(int i=Im.Dim();i--;)
          it_im[i] = C[i];
      }
      while (it_seg!=seg.end());
    }
    while(sig_b!=b);
    return;
  }

  if (type==cm_draw_all_single)
  {
    sig_b = b = m_1(v);
    do
    {
      sig_b = sig(sig_b);
      m_b = alph(sig_b);
      if (v==m(m_b) && sig_b<m_b)
        continue;
      seg=give_segment(sig_b);
      it_seg = seg.first();
      do
      {
        pyr->give_pixel(*it_seg++,p);
        it_im.move_to(p.y, p.x);
        for(int i=Im.Dim();i--;)
          it_im[i] = C[i];
      }
      while (it_seg!=seg.end());
    }
    while(sig_b!=b);
    return;
  }

  sig_b = b = m_1(v);
  do
  {
    sig_b = sig(sig_b);
    seg=give_segment(sig_b);
    it_seg = seg.first();
    do
    {
      pyr->give_pixel(*it_seg++,p);
      it_im.move_to(p.y, p.x);
      for(int i=Im.Dim();i--;)
	it_im[i] = C[i];
    }
    while (it_seg!=seg.end());
  }
  while(sig_b!=b);
}

combinatorial_map::segment::segment():pyr(NULL),level(0),b_first(0),b_last(0)
{}
combinatorial_map::segment::segment(dart first, dart last, int lev, pyramid *p):pyr(p),level(lev),b_first(first),b_last(pyr->give_alpha_base(last))
{}

combinatorial_map::segment::iterator::iterator(segment *s, position p):seg(s)
{
  if (p==first_dart)
  {
    b_current = seg->b_first;
    return;
  }
  if (p==last_dart)
  {
    b_current = seg->b_last;
    return;
  }
  b_current = seg->pyr->give_next_border_dart(seg->b_last,seg->level);
}

combinatorial_map::segment::iterator &combinatorial_map::segment::iterator::operator=(const iterator &it)
{
  seg=it.seg;
  b_current=it.b_current;
  return *this;
}
combinatorial_map::segment::iterator &combinatorial_map::segment::iterator::operator++()
{
  b_current = seg->pyr->give_next_border_dart(b_current,seg->level);
  return *this;
}
combinatorial_map::segment::iterator &combinatorial_map::segment::iterator::operator++(int anything)
{
  res = *this;
  b_current = seg->pyr->give_next_border_dart(b_current,seg->level);
  return res;
}
combinatorial_map::segment::iterator &combinatorial_map::segment::iterator::operator--()
{
  b_current = seg->pyr->give_alpha_base(b_current);
  b_current = seg->pyr->give_next_border_dart(b_current,seg->level);
  b_current = seg->pyr->give_alpha_base(b_current);
  return *this;
}
combinatorial_map::segment::iterator &combinatorial_map::segment::iterator::operator--(int anything)
{
  res = *this;
  b_current = seg->pyr->give_alpha_base(b_current);
  b_current = seg->pyr->give_next_border_dart(b_current,seg->level);
  b_current = seg->pyr->give_alpha_base(b_current);
  return res;
}
move_type combinatorial_map::segment::iterator::give_move()
{
  return seg->pyr->give_move_base(b_current);
}
void combinatorial_map::segment::iterator::give_point(point &p)
{
  seg->pyr->give_pointel(b_current,p);
}
bool combinatorial_map::segment::iterator::give_pixel(point &p1,point &p2)
{
  bool error=true;
  try
  {
    seg->pyr->give_pixel(b_current,p1);
  }
  catch (point_error)
  {
    p1.x = -1;
    p1.y = -1;
    error=false;
  }
  try
  {
    seg->pyr->give_pixel(seg->pyr->give_alpha_base(b_current),p2);
  }
  catch (point_error)
  {
    p2.x = -1;
    p2.y = -1;
    return false;
  }
  return error;
}


bool combinatorial_map::segment::iterator::operator==(const iterator &iter) const
{
  return ( b_current==iter.b_current );
}
bool combinatorial_map::segment::iterator::operator!=(const iterator &iter) const
{
  return ( b_current!=iter.b_current );
}



combinatorial_map::segment::iterator &combinatorial_map::segment::first()
{
  res=iterator::iterator(this);
  return res;
}
combinatorial_map::segment::iterator &combinatorial_map::segment::end()
{
  res=iterator::iterator(this,combinatorial_map::segment::end_dart);
  return res;
}
combinatorial_map::segment::iterator &combinatorial_map::segment::last()
{
  res=iterator::iterator(this,combinatorial_map::segment::last_dart);
  return res;
}
