/** \file equivalent_preserve_edge.hh
    \brief File used to determine if two vertices are equivalent */

#ifndef __EQUIVALENT_PRESERVE_EDGE_HH__
#define __EQUIVALENT_PRESERVE_EDGE_HH__

#include "equivalent_mont_jolion.hh"
#include "vertex_data_color.hh"
#include "border_data_deriche.hh"

/** Model class used to determine if two vertices are equivalent */
class equivalent_preserve_edge: public equivalent_mont_jolion
{
private:
  // Grey Difference allowed beetween two vertices
  double grey_threshold;
  // maximum of the gradient allowed beetween two vertices
  double grad_threshold;
public:
  /// Constructor
  equivalent_preserve_edge(const vertex_data_color &v, const border_data_deriche &b):equivalent_mont_jolion(v, b),grey_threshold(220)
  {
    grad_threshold=0.9*(((border_data_deriche *)b_data)->give_max_gradient());
  }
  /// Function to know if the two vertices must be merged
  bool lambda(int,vertex,vertex,dart,dart);
  /// Function to compute the surviving criterion of a vertex
  /// That will be minimized during the reduction process
  double value(vertex);
  /// Function to compute the merging interset of between two vertices
  /// That will be minimized during the reduction process
  double interest(int,vertex,vertex,dart,dart);


  double give_grey_threshold()
  {
    return grey_threshold;
  }
  double give_grad_threshold()
  {
    return grad_threshold;
  }
  double set_grey_threshold(const double &d)
  {
    return grey_threshold=d;
  }
  double set_grad_ratio(const double &d)
  {
    return grad_threshold=d*(((border_data_deriche *)b_data)->give_max_gradient());
  }
};

#endif // __EQUIVALENT_PRESERVE_EDGE_HH__
