/** \file kernel.hh
    \brief File to compute and hold the survivor kernel of the combinatorial map */

#ifndef __KERNEL_HH__
#define __KERNEL_HH__

#include "equivalent.hh"

/** Model class to compute and hold the survivor kernel of the combinatorial map*/
class kernel
{
protected:
  // Array of bool to mark the dart that must be contracted
  tab_sym_bool contracted;
  // Number of contracted darts
  int nb_contracted;
  // Array of vertex that store the father of each vertex
  tab_label Father;
  // Current level of the pyramid
  int level;
public:
  /// Constructor
  kernel():nb_contracted(0),level(1)
  {}
  /// Destructor
  virtual ~kernel()
  {}
  /// Virtual function to initialize the data befor the computation of the kernel
  virtual void init(int lev=1);
  /// Function to compute the kernel of the combinatorial map
  /// used in a do-compute_kernel while(continue_computation())
  virtual void compute_kernel(combi_map_priv &)=0;
  /// Function to know if the compute_kernel function must be iterate an other time
  /// used in a do-compute_kernel while(continue_computation())
  virtual bool continue_computation()=0;

  /// Function to know if a dart is a contracted one
  bool is_marked(dart);
  /// Give the number of survivor
  int cardinal();
  // adds one dart to the kernel
  void contract_edge(dart,dart);
  /// initialises each vertex as its own father
  void set_father(combi_map_priv&);
  /// Access to the father of one vertex
  virtual vertex father(vertex);
  /// Give the root of the tree containing the vertex
  virtual vertex find_compress(vertex);  

};

#endif // __KERNEL_HH__
