#include "level_set.hh"
#include <assert.h>

static level_set::iterator res;

//*** public ***//

level_set::level_set(const level_set &l):max_level(l.max_level),
					 nb_dart(l.nb_dart),
					 levels(l.levels)
{
  int nb_edge=nb_dart>>1;
  dart_index = new int[l.nb_dart+1];
  assert(dart_index);
  memcpy(dart_index, (l.dart_index-nb_edge), (l.nb_dart+1)*sizeof(int));
  dart_index+=nb_edge;
  darts = new dart[l.nb_dart];
  assert(darts);
  memcpy(darts, l.darts, l.nb_dart*sizeof(dart));
}
level_set::level_set(int nb_dart):max_level(1),nb_dart(nb_dart)
{
  int cpt1=1, nb_edge=nb_dart>>1;
  int i_nb_edge=nb_edge;

  dart_index = new int[nb_dart+1];
  assert(dart_index);
  dart_index+=nb_edge;
  darts = new dart[nb_dart];
  assert(darts);

  for(int i=0;i<nb_edge;i++)
  {
    dart_index[cpt1]=i;
    dart_index[-cpt1]=i_nb_edge;
    darts[i]=cpt1;
    darts[i_nb_edge]=-cpt1;
    i_nb_edge++;
    cpt1++;
  }

  levels[0]=nb_dart;
  levels[1]=0; 
}
level_set::~level_set()
{
  int nb_edge=nb_dart>>1;
  if (darts!=NULL)
    delete[]darts;
  if (dart_index!=NULL)
    delete[](dart_index-nb_edge);
  max_level = 0;
  nb_dart = 0;
}

level_set & level_set::operator=(const level_set &l)
{
  int nb_edge=nb_dart>>1;
  max_level = l.max_level;
  nb_dart = l.nb_dart;
  levels = l.levels;
  if (dart_index!=NULL)
    delete [] (dart_index-nb_edge);
  nb_edge=nb_dart>>1;
  dart_index = new int[l.nb_dart+1];
  assert(dart_index);
  memcpy(dart_index, (l.dart_index-nb_edge), (l.nb_dart+1)*sizeof(int));
  dart_index+=nb_edge;
  if (darts!=NULL)
    delete [] (darts);
  darts = new dart[l.nb_dart];
  assert(darts);
  memcpy(darts, l.darts, l.nb_dart*sizeof(dart));
  return *this;
}

int level_set::load(ifstream &in)
{
  int i, nb_edge=nb_dart>>1;
  in >> max_level;
  //  levels[i]=0;
  for(int i=0; i<max_level; i++)
    in >> levels[i];
  in.get();

  nb_dart=levels(0);
  if (darts!=NULL)
    delete [] darts;
  darts = new dart[nb_dart];
  assert(darts);
  in.read((char *)darts,(int)(nb_dart*sizeof(int)));

  if (dart_index!=NULL)
    delete [] (dart_index-nb_edge);
  nb_edge=nb_dart>>1;
  dart_index = new int[nb_dart+1];
  assert(dart_index);
  dart_index+=nb_edge;
  for(i=0;i<nb_dart;i++)
    dart_index[darts[i]]=i;
  compute_all_levels();

  return 1;
}

int level_set::save(ofstream &out)
{
  out << max_level << endl;
  for(int i=0; i<max_level; i++)
    out << levels(i) << endl;

  out.write((char *)darts,(int)(nb_dart*sizeof(int)));
  return 1;
}

int level_set::give_max_level()
{
  return max_level;
}
void level_set::new_level()
{
  max_level++;
  levels[max_level]=0;
  levels[max_level-1]=levels(max_level-2);
}


int level_set::give_level(dart d)
{
  int res;
  if ( (res=dart_level(d))==0 )
    return max_level;
  return res;
}



void level_set::suppress_dart(dart d)
{
  int b,ind=dart_index[d],level=levels(max_level-1)-1;
  dart_level[d]=max_level-1;

  if(ind == level)
  {
    levels[max_level-1]=level;
    return;
  }

  b=darts[level];
  darts[level]=d;
  darts[ind]=b;
  
  dart_index[b]=ind;
  dart_index[d]=level;

  levels[max_level-1]=level;
}





level_set::iterator &level_set::first(int level)
{
  res=iterator::iterator(*this,level);
  return res;
}
level_set::iterator & level_set::begin(int level)
{
  res=iterator::iterator(*this,level,level_set::begin_dart);
  return res;
}
level_set::iterator &level_set::end(int level)
{
  res=iterator::iterator(*this,level,level_set::end_dart);
  return res;
}

level_set::iterator &level_set::last(int level)
{
  res=iterator::iterator(*this,level,level_set::last_dart);
  return res;
}









//*** private ***//

void level_set::compute_all_levels()
{
  int min_ind,max_ind,index;

  for(int level=1;level<max_level;level++)
  {
    min_ind=levels(level);
    max_ind=levels(level-1);
    for(int i=min_ind;i<max_ind;i++)
    {
      index=darts[i];
      dart_level[index]=level;
    }
  }
  min_ind=levels(max_level);
  max_ind=levels(max_level-1);
  for(int i=min_ind;i<max_ind;i++)
  {
    index=darts[i];
    dart_level[index]=0;
  }
}

void level_set::display()
{
  int nb_edge=nb_dart>>1;
  cout << "nb_edge: " << nb_edge << ", nb_dart: " << nb_dart << endl;
  cout << "darts : ";
  for(int i=0;i<nb_dart;i++)
  {
    if (darts[i]>=0)
      cout << " ";
    if (darts[i]<10)
      cout << " ";
    cout << darts[i] << " ";
  }
  cout << endl;
  cout << "index : ";
  for(int i=-nb_edge;i<=nb_edge;i++)
  {
    if (i!=0)
    {
      if (dart_index[i]>=0)
    	cout << " ";
      if (dart_index[i]<10)
    	cout << " ";
      cout << dart_index[i] << " ";
    }
  }
  cout << endl;

  cout << "level : ";
  for(int i=-nb_edge;i<=nb_edge;i++)
  {
    if (i!=0)
    {
      cout << " " << i << "(" << give_level(i) << ") ";
    }
  }
  cout << endl;


  cout << "verif : ";
  for(int i=-nb_edge;i<=nb_edge;i++)
  {
    if (i!=0)
    {
      if (darts[dart_index[i]]>=0)
    	cout << " ";
      if (darts[dart_index[i]]<10)
    	cout << " ";
      cout << darts[dart_index[i]] << " ";
    }
  }
  cout << endl;

}









/******************************************************************/
/*                         class iterator                         */
/******************************************************************/

//*** public ***//

level_set::iterator::iterator():current(0),obj(0),level(0)
{
}
level_set::iterator::iterator(level_set&l_set,int l,position pos):obj(&l_set),level(l)
{
  if(pos == level_set::first_dart)
    {
      current = l_set.levels(obj->max_level);
      return;
    }
  if(pos == level_set::begin_dart)
    {
      current = l_set.levels(level);
      return;
    }
  if(pos == level_set::last_dart)
    {
      current = l_set.levels(level-1);
      current--;
      return;
    }
  if(pos == level_set::end_dart)
    {
      current = l_set.levels(level-1);
      return;
    }
}
level_set::iterator::iterator(const iterator &iter):current(iter.current),obj(iter.obj),level(iter.level)
{
}
level_set::iterator & level_set::iterator::operator=(const iterator &iter)
{
  current=iter.current;
  obj=iter.obj;
  level=iter.level;
  return *this;
}
dart & level_set::iterator::operator *()
{
  if( current == obj->levels(level-1) )
    throw l_set_iteratorator_error("level_set::iterator op *() out of the level_set");
  return obj->darts[current];
}

level_set::iterator &level_set::iterator::operator++()
{
  current++;
  return *this;
}
level_set::iterator &level_set::iterator::operator++(int nothing)
{
  res=*this;
  current++;
  return res;
}

bool level_set::iterator::operator<(const iterator &iter) const
{
  return ( (level==iter.level) && (current<iter.current) );
}
bool level_set::iterator::operator>(const iterator &iter) const
{
  return ( (level==iter.level) && (current>iter.current) );
}
bool level_set::iterator::operator<=(const iterator &iter) const
{
  return ( (level==iter.level) && (current<=iter.current) );
}
bool level_set::iterator::operator>=(const iterator &iter)const
{
  return ( (level==iter.level) && (current>=iter.current) );
}
bool level_set::iterator::operator==(const iterator &iter) const
{
  return ( (level==iter.level) && (current==iter.current) );
}
bool level_set::iterator::operator!=(const iterator &iter) const
{
  return ( (level==iter.level) && (current!=iter.current) );
}



