/** \file level_set.hh
    \brief File to encoding of the level of each dart in the pyramid */

#ifndef __LEVEL_SET_HH__
#define __LEVEL_SET_HH__

#include <tab_label.hh>
#include <tab_dart.hh>
#include <dart.hh>
#include <iostream>
#include <fstream>
using namespace std;

class l_set_iteratorator_error
{
public:
  l_set_iteratorator_error(){}
  l_set_iteratorator_error(const char* messg)
  {
    cerr << messg<<endl;
  }
};

/** Encoding of the level of each dart in the pyramid **/
class level_set
{
protected:
  // Number of the last level of the level_set
  int max_level;
  // Number of darts stocked on the level_set, 2 times nb_edge
  int nb_dart;
  // Infinit array that hold for each level the first dart index of the level
  tab_label levels;
  // Array to store the darts index
  int *dart_index;
  // Array to store the darts
  dart *darts;
  // Array to store the level of each darts
  tab_dart dart_level;

public:
  /// Copy constructor
  level_set(const level_set &);
  /// Constructor with parameters, the number of darts
  level_set(int);
  /// Constructor
  level_set():max_level(0), nb_dart(0), dart_index(NULL), darts(NULL)
  {}
  /// Destructor
  ~level_set();

  /// Reaffectation operator
  level_set & operator=(const level_set &);

  /// Set the level_set from a flow
  int load(ifstream &in);
  /// Save the level_set into a flow
  int save(ofstream &out);
  
  /// Give the maximum level
  int give_max_level();
  /// Add a new level to the level_set
  void new_level();

  /// Give the level of the dart
  int give_level(dart);
  // Suppress one dart
  void suppress_dart(dart);
  
  typedef enum {first_dart,begin_dart,last_dart,end_dart} position;
  friend class iterator
  {
  protected:
    // current index of the iterator
    int current;
    level_set* obj;
    int level;
    
  public:
    iterator();
    iterator(level_set& l,int level,position pos=level_set::first_dart);
    iterator(const iterator &);

    dart & operator *();

    iterator & operator=(const iterator &);
    iterator & operator++();
    iterator & operator++(int);

    bool operator<(const iterator&) const;
    bool operator>(const iterator&) const;
    bool operator<=(const iterator&) const;
    bool operator>=(const iterator&)const;
    bool operator==(const iterator&) const;
    bool operator!=(const iterator&) const;
  };
  
  /// first dart of the level_set
  iterator & first(int);
  /// first dart of the level
  iterator & begin(int);
  /// after the last dart of the level (loop : for(i=level_set.first();i<level.end();i++)
  iterator & end(int);
  /// last dart of the level (loop : for(i=level_set.first();i<=level.last();i++)
  iterator & last(int);

private:
  // Compute the dart_level that give the level of each dart
  void compute_all_levels();
  void display();
};

typedef level_set::iterator dart_iterator;


#endif // __LEVEL_SET_HH__
