#include <iostream>
#include <format_ppm.hh>
#include <format_pgm.hh>
#include <image.hh>

#include "deriche.hh"
#include "pyramid.hh"
#include "combinatorial_map.hh"
#include "water_value.hh"
#include "water_vertex_data.hh"
#include "watershed.hh"
#include "rm_watersheds.hh"
#include "dynamic_kernel.hh"
#include "waterfall.hh"
#include "export.hh"
using namespace std;


ppm_format format_ppm(0);
pgm_format format_pgm(1);

void inverse_image(image & img)
{
  image::iterator end(img.end());

  for(image::iterator ite(img);
      ite<end;
      ite++)
    {
      *ite=255-*ite;
    }
}

void threshold_gradient(image& grad, int threshold)
{
  
  for(image::iterator ite(grad);
      ite<grad.end();
      ite++)
    {
      if(*ite < threshold)
	*ite=0;
    }
}
void instruction_error(){
	cout << "Error in the interpretation of the instruction." << endl;
	cout << "For more details, use the -help option." << endl;
	exit(0);
}

int main(int argc,char* argv[])
{
  double alpha=atof(argv[1]);
  image grad,img(argv[2]);
  combinatorial_map combi_map_level;
 

    init_deriche_deriv(alpha,img);
    grad=deriche_deriv();
  //  inverse_image(grad);
  //   threshold_gradient(grad,50);
  grad.save("tutu.ppm");
  close_deriche_deriv();
 
  // initialisation of the data
  water_vertex_data v_value(grad,img);
  water_value values(v_value,grad);
  
  /// kernel used to compute watersheds
  watershed water_kernel(values);
  // the pyramid
  pyramid P(img.Width(),img.Height(),values);

  
  //computing watersheds
  P.compute_pyramid(water_kernel);
  values.set_mode(rm_watershed);

  // remove watershed pixels
  rm_watersheds remove_border_points(water_kernel);
  P.compute_pyramid(remove_border_points);  
  //  cerr << "Computing pass values"<<endl;
  values.set_mean(P);

  //  cerr<<"Computing waterfalls"<<endl;
  // apply one step of the  watefall algorithm
  //  waterfall fall(values,30);



  /*  unsigned char C[3]={0,255,0};
  P.give_combinatorial_map(P.give_max_level(),combi_map_level);  
  image res(img);
  combi_map_level.draw_border(res,C);
  res.save("titi.ppm");*/
  //  values.set_mode(cmp_waterfall);
  //  P.compute_pyramid(fall);
  values.set_mode(cmp_dynamic);

  
  // compute contour dynamics

  // compute all remaining levels

  dynamic_kernel dynamics(values,grad,grad,35);
  P.compute_pyramid(dynamics);

  //  P.save("pyramid.pyr");
    cerr << P.give_max_level()<< "levels computed"<<"("<<(P.give_max_level()-1)/3<< "real ones)"<<endl;
    
    
    
  combinatorial_map map;
  P.give_combinatorial_map(P.give_max_level(),map);
  for(vertex_iterator it(map.give_vertex_first());it<map.give_vertex_end();it++)
    {
    }      
  
  
  //      export_to_descartes(combi_map_level,name);
  //  image res(img);
  //  combi_map_level.draw_border(res,C);
  //  res.save("res70.ppm");
  /*  nb_level=P.give_max_level();
  for(int i=7;i<=nb_level;i+=3)
    {
      combinatorial_map map;
      image res(img);
      sprintf(name,"res%d.ppm",i);
      P.give_combinatorial_map(i,combi_map_level);  
      //      export_to_descartes(combi_map_level,name);
      combi_map_level.draw_border(res,C);
      res.save(name);
    }
  */
  //cout << res;

  cout << "This is the End" << endl;
  return 0;
}
