#include "pyramid.hh"
#include "combi_map_base_exp.hh"
#include "combi_map_base_imp.hh"
#include "kernel.hh"
#include "combi_map_base.hh"



pyramid::pyramid(int dimx, int dimy, value_set &val, construction c):map(*this), l_set(2*(dimx*(dimy+1)+dimy*(dimx+1))), value(&val)
{  
  if (c==implicit_base)
    base = new combi_map_base_imp(dimx, dimy);

  else
    base = new combi_map_base_exp(dimx, dimy);
  base->init_combi_map_priv(map);
  value->init(*base);
}
pyramid::pyramid(char *filename, construction c):base(NULL), map(*this), value(NULL)
{
  ifstream in(filename);
  if (!in)
    throw;

  load(in, c);
  in.close();
}
pyramid::pyramid(ifstream &in, construction c):base(NULL), map(*this), value(NULL)
{
  load(in, c);
}
pyramid::~pyramid()
{
  if (base!=0)
    delete [] base;
}
void pyramid::compute_level(kernel &kern)
{
  list_label empty_self_loops;
  value->set_combinatorial_map(map);
  kern.compute_kernel(map);

  if(kern.cardinal()==0)
    return;
  l_set.new_level();
  contract_edge(kern, empty_self_loops);
  l_set.new_level();
  remove_empty_self_loops(empty_self_loops);
  l_set.new_level();
  remove_double_edges(kern);
  update_vertex_set(kern);
}
void pyramid::compute_pyramid(kernel &kern)
{
  do
  {
    compute_level(kern);
  }
  while(kern.continue_computation());
}

int pyramid::load(ifstream &in, construction c)
{
  int level_max, dx, dy, i;
  dart b, sig_b, alpha, from_background;

  // init the combinatorial map of the base of the pyramid
  in >> dx;
  in >> dy;
  if (base!=NULL)
    delete base;
  if (c==implicit_base)
    base = new combi_map_base_imp(dx, dy);
  else
    base = new combi_map_base_exp(dx, dy);
    
  // set the level_set of the map
  l_set.load(in);

  // Update the sigma and alpha of the map
  level_max=l_set.give_max_level();
  level_set::iterator it=l_set.begin(level_max);
  while(it<l_set.end(level_max))
  {
    b = *it;
    sig_b=update_sigma_alpha(b,alpha,level_max,from_background);
    map.alph[b] = alpha;
    map.alph[alpha] = b;
    map.sig[b]=sig_b;
    it++;
  }

  // Update the mu mu_1 and the list of vertices of the map
  // For the background vertex
  sig_b = b = from_background;
  i = dx*dy+1;
  map.vertex_set.insert(i);
  do
  {
    sig_b = map.sigma(sig_b);
    map.m[sig_b] = i;
  }
  while( sig_b!=b );
  map.m_1[i] = sig_b;
  
  // For the others vertices
  i = 0;
  it=l_set.begin(level_max);
  while(it<l_set.end(level_max))
  {
    sig_b = b = *it;
    it++;
    if (map.m(b)!=0)
      continue;
    map.vertex_set.insert(++i);
    do
    {
      sig_b = map.sigma(sig_b);
      map.m[sig_b] = i;
    }
    while( sig_b!=b );
    map.m_1[i] = sig_b;
  }
  return 1;
}
int pyramid::save(const char *filename)
{
  ofstream out(filename);
  if (!out)
    return -1;

  out << base->give_width() << endl;
  out << base->give_height() << endl;
  l_set.save(out);
  out.close();
  return 1;
}

void pyramid::give_combinatorial_map(int level, combinatorial_map &c_map)
{
  dart b, alpha, sig_b, alpha_sig, sig_end;
  int i=0, ind_or=0, current_ind=0, background;
  tab_dart index_or;
  tab_dart val;
  c_map.sig.reinit();
  c_map.alph.reinit();
  c_map.m.reinit();
  c_map.m_1.reinit();
  c_map.vertex_set.reinit();

  c_map.pyr=this;
  c_map.open_parenthesis.reinit();
  c_map.nb_edge = map.give_nb_edge();
  background=c_map.background=map.give_background();
  c_map.level=level;


  // Update the background
  b = sig_end = map.m_1(map.give_background());
  c_map.vertex_set.insert(c_map.background);
  do
  {
    c_map.m[b] = c_map.background;
    sig_b=extract_update_sigma_alpha(b,alpha,level,ind_or,c_map.alph);
    index_or[b] += ind_or;
    index_or[alpha] -= ind_or;
    c_map.alph[b] = alpha;
    c_map.alph[alpha] = b;
    c_map.sig[b]=sig_b;
    b = sig_b;
  }
  while( sig_end!=b );
  c_map.m_1[background] = b;

  // Update the rest
  level_set::iterator it=l_set.first(level);
  while(it<l_set.end(level))
  {
    sig_end = b = *it;
    it++;
    if (c_map.m(b)!=0)
      continue;
    c_map.vertex_set.insert(++i);
    do
    {
      // Update the parametres
      c_map.m[b] = i;
      sig_b=extract_update_sigma_alpha(b,alpha,level,ind_or,c_map.alph);
      index_or[b] += ind_or;
      index_or[alpha] -= ind_or;
      c_map.alph[b] = alpha;
      c_map.alph[alpha] = b;
      c_map.sig[b]=sig_b;
      b = sig_b;
    }
    while( sig_end!=b );
    c_map.m_1[i] = b;
  }

  // Mark the open parenthesis
  int tmp;
  list_label::iterator v_it=c_map.vertex_set.first();
  while(v_it<c_map.vertex_set.last())
  {
    i = *++v_it;
    sig_end = c_map.m_1(i);
    b = c_map.sig(sig_end);
    do
    {
      sig_b = c_map.sig(b);
      alpha = c_map.alph(b);
      if (c_map.open_parenthesis(sig_b)==true)
      {
        alpha_sig=c_map.alph(sig_b);
	val[current_ind  ] += index_or(b);
	val[current_ind-1] += val(current_ind)-base->compute_index_orientation(alpha, sig_b);
	val[current_ind  ] += -base->compute_index_orientation(alpha,c_map.sig(alpha_sig));
	b = sig_b;
        tmp = val[current_ind];
        val[current_ind--]=0;
	if (tmp!=4)
	  continue;
	c_map.open_parenthesis.light(alpha_sig);
	c_map.open_parenthesis.unlight(sig_b);
	continue;
      }
      val[current_ind] += index_or(b)-base->compute_index_orientation(alpha, sig_b);
      if (c_map.m(alpha)!=i || c_map.open_parenthesis(b)==true || c_map.open_parenthesis(alpha)==true)
      {
        b=sig_b;
	continue;
      }
      c_map.open_parenthesis.light(alpha);
      current_ind++;
      b=sig_b;
      continue;
    }
    while( sig_end!=b );
    val[current_ind]=0;
  }




}

int pyramid::give_max_level()
{
  return l_set.give_max_level();
}

dart pyramid::give_next_border_dart(dart d, int level)
{
  dart b;
  int level_b;
  b=base->alpha(d);

  do
  {
    b=base->phi(b);
    level_b=l_set.give_level(b);
  }
  while(level_b%3!=0 && level_b<level);
  return b;
}

void pyramid::contract_edge(kernel &kern, list_label &empty_self_loops)
{
  dart b, mb, b_tmp;
  vertex v_b, v_mb;
  int level_max=l_set.give_max_level();
  list_label d_list;

  level_set::iterator it=l_set.begin(level_max);
  while(it<l_set.end(level_max))
  {
    b = *it;
    it++;
    mb = map.alpha(b);
    v_b = map.m(b);
    v_mb = map.m(mb);
    // we check if the dart must be contracted
    if (kern.is_marked(b))
    {
      // Then, we do the contraction on the value
      value->contract_edge(level_max, map,b, mb, v_b, v_mb);
      // We put the two darts to be supress in the list
      d_list.insert(b); 
      d_list.insert(mb);
      continue;
    }
    // If it is a surviving dart, we update the sigma permutation
    b_tmp = compute_sigma_successor(kern, b);
    // We check for empty self-loop
    if (b_tmp!=mb)
      continue;
    empty_self_loops.insert(b);
  }
  
  // Then, we supress the darts
  list_label::iterator iter(d_list.first());
  while(iter<d_list.end())
    l_set.suppress_dart(*(iter++));
}
void pyramid::remove_empty_self_loops(list_label &empty_self_loops)
{
  if(empty_self_loops.empty())
    return;
  dart b, b_prev, sig_b, sig_sig_b;

  list_label::iterator iter(empty_self_loops.first());
  while(iter<empty_self_loops.end())
  {
    b_prev = *(iter++);
    sig_sig_b = map.sigma(b_prev);
    do
    {
      b = b_prev;
      b_prev = map.sigma_1(b);
      sig_b = sig_sig_b;
      sig_sig_b = map.sigma(sig_b);
      value->remove_empty_self_loops(b, sig_b, map.m(b));
      l_set.suppress_dart(b);
      l_set.suppress_dart(sig_b);
    }
    while(sig_sig_b==map.alpha(b_prev));
    map.sig[b_prev]=sig_sig_b;
  }
}
void pyramid::remove_double_edges(kernel &kern)
{
  dart b;
  int level = l_set.give_max_level();
  tab_sym_bool suppressed;
  list_label double_edge;

  level_set::iterator iter(l_set.begin(level));
  while(iter<l_set.end(level))
  {
    b = *(iter++);
    if (suppressed(b))
      continue;
    if (map.phi(map.phi(b))==b)
      continue;
    update_sigma_double_edge(kern, double_edge, suppressed, b);
  }

  list_label::iterator list_iter(double_edge.first());
  while(list_iter<double_edge.end())
  {
    b = *(list_iter++);
    l_set.suppress_dart(b);  
  }
}
void pyramid::update_vertex_set(kernel &kern)
{
  dart b;
  vertex v, f_v;
  int level = l_set.give_max_level();
  tab_bool survived_vertices;

  // Go through all the darts and update the mu and mu_1 array
  level_set::iterator iter(l_set.begin(level));
  while(iter<l_set.end(level))
  {
    b = *(iter++);
    v = map.m(b);
    f_v=kern.find_compress(v);
    value->update_vertex_owner(b,v,f_v);
    map.m[b] = f_v;
    if (survived_vertices(f_v))
      continue;
    survived_vertices.light(f_v);
    map.m_1[f_v] = b;
  }

  // Remove the merged vertex from the vertex list of the combinatorial map
  list_label::iterator list_iter(map.vertex_set.first());
  while(list_iter<map.vertex_set.end())
  {
    v = *list_iter;
    if ((unsigned)v==map.give_background())
    {
      list_iter++;
      continue;
    }
    f_v = kern.find_compress(v);
    value->update_vertex_set(level, v, f_v);
    if (!survived_vertices(v))
    {
      map.vertex_set.remove(list_iter);
      continue;
    }
    list_iter++;
  }
}

void pyramid::update_sigma_double_edge(kernel &kern, list_label &double_edge, tab_sym_bool &suppressed, dart b)
{
  int level = l_set.give_max_level();
  dart current,tmp, bp=map.sigma(b), alpha=map.alpha(b);



  // computation of alpha(b) and marking of suppressed darts
  while(map.phi(map.phi(bp))==bp)
  {

    tmp = map.alpha(bp);
    
    double_edge.insert(bp);
    suppressed.light(bp);
    double_edge.insert(alpha);
    suppressed.light(alpha);
    alpha=tmp;
    bp=map.sigma(bp);
  }
  //  update of the values from alpha(b) to b
  current=alpha;
  while(map.alpha(current) != b)
    {
      value->remove_double_edges(level, b,map.alpha(current),alpha);
      current=map.sigma(current);
      value->remove_double_edges(level, alpha, current,b);
    }


  // update of the permutation
  map.sig[b]=bp;
  map.sig[alpha]=map.sig[map.alph[b]];
  map.alph[b]=alpha;
  map.alph[alpha]=b;
}

dart pyramid::compute_sigma_successor(kernel &kern, dart b)
{
  dart b_succ=map.sigma(b);
  while(kern.is_marked(b_succ)||kern.is_marked(map.alph(b_succ)))
    b_succ=map.sigma(map.alph(b_succ));
  return map.sig[b]=b_succ;
}

dart pyramid::update_sigma_alpha(dart d, dart &alpha, int current_level, dart &from_background)
{
  dart b=d;
  bool boul=false;
  int level;
  do
    {
      b=update_next_dart(b, alpha, current_level, boul);
      level=l_set.give_level(b);
    }
  while(level < current_level);
  if (boul)
    from_background = alpha;
  return b;
}
dart pyramid::update_next_dart(dart d, dart &alpha, int current_level, bool &from_background)
{
  int level_b;
  dart b;
  if( (b=map.alpha(d)) == 0)
    b=base->alpha(d);
  if (l_set.give_level(b) >= current_level)
    alpha = b;

  do
  {
    if (b == -1)
      from_background = true;
    b=base->alpha(b);
    b=base->sigma(b);
    level_b=l_set.give_level(b);
  }
  while((level_b%3 != 0)&&(level_b < current_level));

  return b;
}


dart pyramid::extract_update_sigma_alpha(dart b, dart &alpha, int current_level, int &ind_or, tab_dart &alph)
{
  int level;
  dart prev;

  prev = b;
  b=extract_update_next_dart(b, alpha, current_level, alph);
  level=l_set.give_level(b);
  ind_or=0;
  while(level < current_level)
  {
    ind_or += base->compute_index_orientation(prev, b);
    prev = b;
    b=extract_update_next_dart(b, alpha, current_level, alph);
    level=l_set.give_level(b);
  }

  return b;

}
dart pyramid::extract_update_next_dart(dart d, dart &alpha, int current_level, tab_dart &alph)
{
  int level_b;
  dart b;
  if( (b=alph(d)) == 0)
    b=base->alpha(d);
  if (l_set.give_level(b) >= current_level)
    alpha = b;

  do
  {
    b=base->alpha(b);
    b=base->sigma(b);
    level_b=l_set.give_level(b);
  }
  while((level_b%3 != 0)&&(level_b < current_level));

  return b;
}
