#include "vertex_data_color.hh"

static vertex_data_color::vertex_value static_vertex_value;

vertex_data_color::vertex_data_color(image &I,quantize &Q):img(&I),quant(&Q)
{
}
vertex_data_color::~vertex_data_color()
{
  int i=img->Width()*img->Height();
  do
  {
    if ( data(i)!=NULL )
    {
      delete (vertex_value *)data[i];
      data[i] = NULL;
    }
  }
  while(--i);
}



vertex_data_color::vertex_value::vertex_value():merged(false),perimetre(4),cardinal(1)
{
  memset(M1, 0, 3*sizeof(double));
  memset(M2, 0, 3*sizeof(double));
}
vertex_data_color::vertex_value::vertex_value(const vertex_value &v):merged(v.merged),perimetre(v.perimetre),cardinal(v.cardinal)
{
  memcpy(M1, v.M1, 3*sizeof(double));
  memcpy(M2, v.M2, 3*sizeof(double));
}
vertex_data_color::vertex_value::vertex_value(int *pix, int img_dim):merged(false),perimetre(4),cardinal(1)
{
  for(int i=img_dim;i--;)
  {
    M1[img_dim]=pix[img_dim];
    M2[img_dim]=pix[img_dim]*pix[img_dim];
  }
}
vertex_data_color::vertex_value & vertex_data_color::vertex_value::operator=(const vertex_value &v)
{
  merged = v.merged;
  perimetre = v.perimetre;
  cardinal = v.cardinal;
  for(int i=3;i--;)
  {
    M1[i]=v.M1[i];
    M2[i]=v.M2[i]*v.M2[i];
  }
  return *this;
}

vertex_data_color::vertex_value vertex_data_color::vertex_value::reinit(double *_M1,double *_M2,int C,int img_dim,int perim)
{
  cardinal=C;
  perimetre=perim;
  merged=false;
  memcpy(M1, _M1, img_dim*sizeof(double));
  memcpy(M2, _M2, img_dim*sizeof(double));
  return *this;
}
void vertex_data_color::vertex_value::give_color(double *C)
{
  memcpy(C, M1, 3*sizeof(double));
}
void vertex_data_color::vertex_value::give_squared_color(double *C)
{
  memcpy(C, M2, 3*sizeof(double));  
}
int vertex_data_color::vertex_value::give_cardinal()
{
  return cardinal;
}
void vertex_data_color::vertex_value::give_mean(double *C)
{
  for(int k=3;k--;)
    C[k]=M1[k]/cardinal;
}
double vertex_data_color::vertex_value::give_variance()
{
  double res=0, tmp;
  for(int k=3;k--;)
  {
    tmp=M1[k]/cardinal;
    res+=M2[k]/cardinal+tmp*tmp;
  }
  return tmp;
}
double vertex_data_color::vertex_value::give_se()
{
  double res=0;
  for(int k=3;k--;)
    res+=M2[k]-M1[k]*M1[k]/cardinal;
  return res;
}
int vertex_data_color::vertex_value::give_perimetre()
{
  return perimetre;
}
int vertex_data_color::vertex_value::add_perimetre(int val)
{
  return perimetre+=val;
}


vertex_data_color::vertex_value &vertex_data_color::operator() (long i) const
{
  vertex_value *pt = (vertex_value *)(data(i));
  if (pt==NULL)
  {
    static_vertex_value.reinit(0,0,0,0,0);
    return static_vertex_value;
  }
  return *pt;
}
void vertex_data_color::del(vertex v)
{
  delete (vertex_value *)data[v];
  data[v]=NULL;
}


void vertex_data_color::init(combi_map_base &map,vertex V)
{
  int width=img->Width(), dim=img->Dim(), x, y;
  double M1[3], M2[3];
  image::iterator it = img->begin();
  y = (V-1)/width;
  x = (V-1)%width;
  it.move_to(y,x);
  for (int i=0;i<dim;i++)
  {
    M1[i] = it[i];
    M2[i] = M1[i]*M1[i];
  }
  data[V] = new vertex_value();
  ((vertex_value *)data[V])->reinit((double *)&M1,(double *)&M2,1,dim,4);
}
void vertex_data_color::merge(int level,vertex V1,vertex V2)
{
  int card, perim, dim=img->Dim();
  double M1[3],M2[3],_M1[3],_M2[3];
  vertex_value *v1, *v2;
  v1 = (vertex_value *)data[V1];
  v2 = (vertex_value *)data[V2];
  card = v1->give_cardinal() + v2->give_cardinal();
  perim = v1->give_perimetre() + v2->give_perimetre();

  v1->give_color(M1);
  v1->give_squared_color(M2);
  v2->give_color(_M1);
  v2->give_squared_color(_M2);
  for(int k=0;k<dim;k++)
    {
      M1[k]+=_M1[k];
      M2[k]+=_M2[k];
    }
  v1->reinit(M1,M2,card,dim,perim);
}
int vertex_data_color::give_label(vertex v)
{
  double D[3];
  int C[3], card;
  ((vertex_value *)(data(v)))->give_color((double *)&D);
  card=((vertex_value *)(data(v)))->give_cardinal();
  for(int i=0;i<3;i++)
    C[i]=(int)(D[i]/card);
  return quant->give_color_index(C);
}
int vertex_data_color::give_dim_color()
{
  return img->Dim();
}
