/** \file vertex_data_color.hh
    \brief File that manage the class which store the perimeter and mean color of each vertex */

#ifndef __VERTEX_DATA_COLOR_HH__
#define __VERTEX_DATA_COLOR_HH__

#include <tab_ptr.hh>
#include <image.hh>
#include <quantize.hh>
#include <string.h>
#include "vertex_data.hh"

/** One class which store the perimeter and mean color of each vertex */
class vertex_data_color: public vertex_data
{
private:
  // Array of pointor on the vertex value
  tab_ptr data;
  // Pointor on the picture that will be used to initialize the vertex_value
  image *img;
  // Pointor on the quantizer of the picture
  quantize *quant;

public:
  /// Constructor with parameters, the number of darts
  vertex_data_color(image &I,quantize &Q);
  /// Destructor
  virtual ~vertex_data_color();

  /** Class that hold all the information of one vertex */
  class vertex_value
  {
  private:
    // Sum of the color of all the vertices which were merged to the vertex
    double M1[3];
    // Sum of the squar of the color of all the vertices which were merged to the vertex
    double M2[3];
    // boolean that show the state of the vertex
    bool merged;
    // perimeter of the area describe by the vertex
    int perimetre;
    // Number of the vertices which were merged to the vertex
    int cardinal;

  public:
    /// Constructor
    vertex_value();
    /// Copy constructor
    vertex_value(const vertex_value &);
    /// Constructor with parameters, a pointor on the vertex color value and the dim of the color array
    vertex_value(int *, int);
    /// Destructor
    ~vertex_value()
    {}
  
    /// Reaffectation operator
    vertex_value & operator=(const vertex_value &);
    /// Reinitialiation function
    /// M1, M2, Cardinal, img_dim, perimetre
    vertex_value reinit(double*,double*,int,int,int);

    /// Access to the color sum of all the vertices merged with the vertex
    void   give_color(double *);
    /// Access to the squared color sum of all the vertices which were merged
    void   give_squared_color(double *);
    /// Access to the cardinal of the vertex (number of vertices merged with it)
    int    give_cardinal();
    /// Give the color mean of the vertex
    void   give_mean(double*);
    /// Give the color varience inside the vertex
    double give_variance();
    /// Give the quadratic error inside the vertex
    double give_se();
    /// Give the perimeter of the vertex with a pixel side as unit
    int    give_perimetre();
    /// Add a value to the perimeter of the vertex
    int    add_perimetre(int);
  };


  /// Access an element of dataset
  vertex_value &operator() (long) const;
  /// Remove an element from the dataset
  void del(vertex);

  /// Function to initialize the vertex_value of the vertex
  void init(combi_map_base &,vertex);
  /// Function to manage the vertex_value during the merge of two vertices
  void merge(int,vertex,vertex);
  
  /// Give the label of the color of one vertex
  int give_label(vertex);
  /// Give the depth of the color
  int give_dim_color();
};

#endif // __VERTEX_DATA_COLOR_HH__
