#include "water_vertex_data.hh"
#include "moments.hh"

#define MIN(a,b) ((a)<(b)?(a):(b))


water_vertex_data::water_vertex_data(image &depths,image& I):grad(&depths),img(&I)
{
  moments::set_dim(I.Dim());
}
void water_vertex_data::init(combi_map_base& map,vertex v)
{
  int width=grad->Width();
  int x, y;
  // gradient information
  image::iterator it = grad->begin();
  y = (v-1)/width;
  x = (v-1)%width;
  it.move_to(y,x);
  depths[v]=*it;
  // image information
  image::iterator it_img = img->begin();
  it_img.move_to(y,x);
  tab_moments[v]=new moments(it_img);
}

void water_vertex_data::merge(int level,
			      vertex v1,
			      vertex v2)
{
  int depth1=depths[v1];
  int depth2=depths[v2];
  moments* m1=(moments*)tab_moments[v1];
  moments *m2=(moments*)tab_moments[v2];
  int depth=MIN(depth1,depth2);

  depths[v2]=depth;
  if(tab_moments[v1])
    {
      m2->merge(*m1);
      delete m1;
    }
}

int  water_vertex_data::give_depth(vertex v) const
{
    return depths(v);
}
void water_vertex_data::set_depth(vertex v,int depth)
{
  depths[v]=depth;
}
