#include <assert.h>
#include "watershed.hh"
#include <fifo_label.hh>
#include <image.hh>

using namespace std;

#include <min_max.hh>

watershed::watershed(water_value& d):data(&d),im0(),imd(),watershed_set(2000)
{
  d.set_labels(im0,WSHED);
}
watershed::~watershed()
{
}

void watershed::compute_kernel(combi_map_priv & map)
{  
#ifdef __DEBUG__
  //    image debug("/home/luc/TRAVAIL/IMAGE/lenna.ppm");
  //   image debug("essai.ppm");
  //  image debug("../neurones.ppm");
  vertex debug_contracted;
  image::iterator debug_ite(debug);
  int debug_iter(0);
#endif
  list_label levels[256];
  int h,h_min(255),h_max(0);
  dart b,mb,b0,contract;
  vertex v,vp,background(map.give_background());
  vertex vp2;
  vertex fictitious(*map.vertex_set.end());
  vertex_iterator v_iter(map.vertex_set);
  vertex_iterator end_vertex_set(map.vertex_set.end());
  int current_dist;
  fifo_label fifo(2000);
  int current_label(0);
  label label_v,label_vp;

  int dist_vp;
  // no dart is initially contracted 
  contracted.reinit();

  Father[background]=background;
  // sort the initial vertices according to their depth
  v_iter=map.vertex_set.first();
  for (;v_iter<end_vertex_set;v_iter++)
    {
      v=(*v_iter);

      if (v== background)
	continue;
      // each vertex is its own father
      Father[v]=v;
      // initialisation of im0 and imd
      im0[v]=INIT;
      imd[v]=0;
      
      h=data->depth(v);
  
      levels[h].insert(v);
      h_min=MIN(h_min,h);
      h_max=MAX(h_max,h);    
    }
  // Construction of the kernel
  for(h=h_min;h<h_max+1;h++)
    {
      cerr << "level : "<<h<<endl;
      for(list_label::iterator iter=levels[h].first();
	  iter<levels[h].end();iter++)
	{
	  v=*iter;
	  if(v==background)
	    continue;
	  
	  im0[v]=MASK;
	  b0=b=map.m_1(v);
	  do
	    {
	      
	      mb=map.alpha(b);
	      vp=map.m(mb);
	      if(vp==background)
		{
		  b=map.sigma(b);
		  continue;
		}
	      if(im0(vp)>0||im0(vp) == WSHED)
		{
		  imd[v]=1;
		  fifo.add(v);
		  break;
		}
	      b=map.sigma(b);
	    }
	  while(b != b0);
	  }
      current_dist=1;
      fifo.add(fictitious);
      do
	{
	  v=fifo.first();

	  if(v==fictitious)
	    {
	      if(fifo.empty())
		break;
	      else
		{
		  fifo.add(fictitious);
		  current_dist++;
		  v=fifo.first();
		}
	    }
	  b0=b=map.m_1(v);
	  label_v=im0(v);
	  contract=0;
	  bool voisin(false);
	  do
	    {	      
	      mb=map.alpha(b);
	      vp=map.m(mb);
	      if(vp==background)
		{
		  b=map.sigma(b);
		  continue;
		}
	      dist_vp=imd(vp);
	      label_vp=im0(vp);
	      if((dist_vp<=current_dist)&& 
		 (label_vp>0)||(label_vp==WSHED))
		 
		{
		  if(label_vp>0)
		    {
		      if((label_v==MASK)||((label_v==WSHED)&&!voisin))
			{
			  im0[v]=label_vp;
			  label_v=label_vp;

			  contract=b;
#ifdef __DEBUG__
			  //			  imd[v]=0;
			  debug_contracted=vp;
#endif
			  Father[v]=find_compress(vp);
			}
		      else	
			{
			  if (label_v != label_vp)
			    {
			      im0[v]=WSHED;
			      voisin=true;
			      label_v=WSHED;
			    }
			}
		    }
		  else
		    if(label_v==MASK)
		      {
			im0[v]=WSHED;
			label_v=WSHED;
		      }
		}
	      else
		{		  
		  
		  if ((label_vp==MASK) && (dist_vp==0))
		    {
		      imd[vp]=current_dist+1;
		      dist_vp++;
		      fifo.add(vp);
		    }
		}
	      b=map.sigma(b);
	    }
	  while(b!=b0);
	  assert(im0(v) != INIT);
	  if(im0(v) != WSHED)
	    {
      contract_edge(contract,map.alpha(contract));
#ifdef __DEBUG__
	      debug_iter++;
		
	      //	      cerr << "contract "<<contract<<","<<map.alpha(contract)<<endl;		      
	      debug_ite.move_to((v-1)/debug.Width(),(v-1)%debug.Width());
	      debug_ite[0]=(im0(v)&0xff);
	      debug_ite[1]=(im0(v)&0xff00);
	      debug_ite[2]=(im0(v)&0xff0000);
#endif
	    }
	  else
	    {
	      watershed_set.add(v);
	      Father[v]=v;
#ifdef __DEBUG__
	      debug_ite.move_to((v-1)/debug.Width(),(v-1)%debug.Width());
	      debug_ite[0]= debug_ite[2]=0;
	      debug_ite[1]=255;
#endif
	      }
	}	  
      while(1);
      for(list_label::iterator iter=levels[h].first();
	  iter<levels[h].end();iter++)
	{
	  v=*iter;
	  if(v==background)
	    continue;
	  imd[v]=0;

	  if(im0(v)==MASK)
	    {
	      current_label++;
	      fifo.add(v);
	      im0[v]=current_label;
#ifdef __DEBUG__
	      debug_ite.move_to((v-1)/debug.Width(),(v-1)%debug.Width());
	      debug_ite[0]=(im0(v)&0xff);
	      debug_ite[1]=(im0(v)&0xff00);
	      debug_ite[2]=(im0(v)&0xff0000);
#endif			  
	      while(!fifo.empty())
		{
		  vp=fifo.first();
		  b0=b=map.m_1(vp);
		  do
		    {
		      mb=map.alpha(b);
		      vp2=map.m(mb);
		      if(vp2==background)
			{
			  b=map.sigma(b);
			  continue;
			}
		      if(im0(vp2) == MASK)
			{
			  fifo.add(vp2);
			  im0[vp2]=current_label;
			  contract_edge(b,mb);
			  //			  cerr << "contract "<<b<<","<<mb<<endl;
			  Father[vp2]=find_compress(vp);

#ifdef __DEBUG__
			  debug_ite.move_to((vp2-1)/debug.Width(),(vp2-1)%debug.Width());
			  debug_ite[0]=(im0(v)&0xff);
			  debug_ite[1]=(im0(v)&0xff00);
			  debug_ite[2]=(im0(v)&0xff0000);
#endif
	      
			}
		      b=map.sigma(b);
		    }
		  while(b!=b0);
		}
	    }
	}
    }

#ifdef __DEBUG__
  debug.save("debug.ppm");
#endif
}
